import 'package:process/process.dart';

import 'base/config.dart';
import 'base/context.dart';
import 'base/error_handling_io.dart';
import 'base/file_system.dart';
import 'base/io.dart';
import 'base/logger.dart';
import 'base/os.dart';
import 'base/platform.dart';
import 'base/process.dart';
import 'base/signals.dart';
import 'base/terminal.dart';
import 'base/time.dart';
import 'util/util.dart';
import 'version.dart';

FlutterVersion get flutterVersion => context.get<FlutterVersion>()!;

String? flutterRoot;

String? faircliRoot;

ProcessUtils get processUtils => context.get<ProcessUtils>()!;

Config get config => context.get<Config>()!;

OperatingSystemUtils get os => context.get<OperatingSystemUtils>()!;

// Unless we're in a test of this class's signal handling features, we must
// have only one instance created with the singleton LocalSignals instance
// and the catchable signals it considers to be fatal.
LocalFileSystem? _instance;

LocalFileSystem get localFileSystem =>
    _instance ??= LocalFileSystem(
      LocalSignals.instance,
      Signals.defaultExitSignals,
      shutdownHooks,
    );

const Platform _kLocalPlatform = LocalPlatform();

Platform get platform => context.get<Platform>() ?? _kLocalPlatform;

/// Currently active implementation of the file system.
///
/// By default it uses local disk-based implementation. Override this in tests
/// with [MemoryFileSystem].
FileSystem get fs =>
    ErrorHandlingFileSystem(
      delegate: context.get<FileSystem>() ?? localFileSystem,
      platform: platform,
    );

const ProcessManager _kLocalProcessManager = LocalProcessManager();

Signals get signals => context.get<Signals>() ?? LocalSignals.instance;

/// The active process manager.
ProcessManager get processManager =>
    context.get<ProcessManager>() ?? _kLocalProcessManager;

Logger get logger => context.get<Logger>()!;

/// Display an error level message to the user. Commands should use this if they
/// fail in some way.
///
/// Set [emphasis] to true to make the output bold if it's supported.
/// Set [color] to a [TerminalColor] to color the output, if the logger
/// supports it. The [color] defaults to [TerminalColor.red].
void printError(String message, {
  StackTrace? stackTrace,
  bool? emphasis,
  TerminalColor? color,
  int? indent,
  int? hangingIndent,
  bool? wrap,
}) {
  logger.printError(
    message,
    stackTrace: stackTrace,
    emphasis: emphasis ?? false,
    color: color,
    indent: indent,
    hangingIndent: hangingIndent,
    wrap: wrap,
  );
}

/// Display a warning level message to the user. Commands should use this if they
/// have important warnings to convey that aren't fatal.
///
/// Set [emphasis] to true to make the output bold if it's supported.
/// Set [color] to a [TerminalColor] to color the output, if the logger
/// supports it. The [color] defaults to [TerminalColor.cyan].
void printWarning(String message, {
  bool? emphasis,
  TerminalColor? color,
  int? indent,
  int? hangingIndent,
  bool? wrap,
}) {
  logger.printWarning(
    message,
    emphasis: emphasis ?? false,
    color: color,
    indent: indent,
    hangingIndent: hangingIndent,
    wrap: wrap,
  );
}

/// Display normal output of the command. This should be used for things like
/// progress messages, success messages, or just normal command output.
///
/// Set `emphasis` to true to make the output bold if it's supported.
///
/// Set `newline` to false to skip the trailing linefeed.
///
/// If `indent` is provided, each line of the message will be prepended by the
/// specified number of whitespaces.
void printStatus(String message, {
  bool? emphasis,
  bool? newline,
  TerminalColor? color,
  int? indent,
  int? hangingIndent,
  bool? wrap,
}) {
  logger.printStatus(
    message,
    emphasis: emphasis ?? false,
    color: color,
    newline: newline ?? true,
    indent: indent,
    hangingIndent: hangingIndent,
    wrap: wrap,
  );
}

/// Display the [message] inside a box.
///
/// For example, this is the generated output:
///
///   ┌─ [title] ─┐
///   │ [message] │
///   └───────────┘
///
/// If a terminal is attached, the lines in [message] are automatically wrapped based on
/// the available columns.
void printBox(String message, {
  String? title,
}) {
  logger.printBox(message, title: title);
}

/// Use this for verbose tracing output. Users can turn this output on in order
/// to help diagnose issues with the toolchain or with their setup.
void printTrace(String message) => logger.printTrace(message);

AnsiTerminal get terminal {
  return context.get<AnsiTerminal>() ?? _defaultAnsiTerminal;
}

final AnsiTerminal _defaultAnsiTerminal = AnsiTerminal(
  stdio: stdio,
  platform: platform,
  now: DateTime.now(),
);

/// The current system clock instance.
SystemClock get systemClock => context.get<SystemClock>() ?? _systemClock;
SystemClock _systemClock = const SystemClock();

/// The global Stdio wrapper.
Stdio get stdio => context.get<Stdio>() ?? (_stdioInstance ??= Stdio());
Stdio? _stdioInstance;

final OutputPreferences _default = OutputPreferences(
  wrapText: stdio.hasTerminal,
  showColor: platform.stdoutSupportsAnsi,
  stdio: stdio,
);

OutputPreferences get outputPreferences =>
    context.get<OutputPreferences>() ?? _default;

ShutdownHooks? get shutdownHooks => context.get<ShutdownHooks>();

const String kDefaultFrameworkChannel = 'master';

/// The flutter GitHub repository.
String get flutterGit =>
    platform.environment['FLUTTER_GIT_URL'] ??
        'https://github.com/flutter/flutter.git';

/// Reads the process environment to find the current user's home directory.
///
/// If the searched environment variables are not set, '.' is returned instead.
String userHomePath() {
  final String envKey =
  platform.operatingSystem == 'windows' ? 'APPDATA' : 'HOME';
  return platform.environment[envKey] ?? '.';
}

/// pub环境下读取.pub-cache/hosted里源码文件夹路径
/// 源码环境下 读取文件路径
String userRootPath() {
  if (cliEnvironment() == CliEnvironment.debug) {
    return fs.path.join(faircliRoot ?? '');
  }
  String fileRoot;
  if (platform.operatingSystem == 'windows') {
    fileRoot = fs.path.join(userHomePath(), 'pub', 'cache', 'hosted',
        'pub.flutter-io.cn', 'fair_cli-$cliVersion');
    if (!fs.directory(fileRoot).existsSync()) {
      fileRoot = fs.path.join(userHomePath(), 'pub', 'cache', 'hosted',
          'pub.dartlang.org', 'fair_cli-$cliVersion');
    }
  } else {
    fileRoot = fs.path.join(userHomePath(), '.pub-cache', 'hosted',
        'pub.flutter-io.cn', 'fair_cli-$cliVersion');
    if (!fs.directory(fileRoot).existsSync()) {
      fileRoot = fs.path.join(userHomePath(), '.pub-cache', 'hosted',
          'pub.dartlang.org', 'fair_cli-$cliVersion');
    }
  }
  return fileRoot;
}
